--[[
Producer - a framework for World of Warcraft professions

Copyright (C) 2006-2008  Prat Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local AceGUI = LibStub("AceGUI-3.0")
local Producer = LibStub( 'AceAddon-3.0'):GetAddon( 'Producer')
local L = LibStub( 'AceLocale-3.0'):GetLocale( 'Producer')
local Snowflake = LibStub( 'Snowflake-2.0')

local CraftFrame = AceGUI:Create( 'ProducerEscape')
Producer.CraftFrame = CraftFrame

local ICONS_PROFESSION  = {
	['Alchemy']		   = 'Interface\\Icons\\Trade_Alchemy',
	['Blacksmithing']  = 'Interface\\Icons\\Trade_BlackSmithing',
	['Cooking']		   = 'Interface\\Icons\\INV_Misc_Food_15',
	['Enchanting']	   = 'Interface\\Icons\\Trade_Engraving',
	['Engineering']	   = 'Interface\\Icons\\Trade_Engineering', 
	['First Aid']	   = 'Interface\\Icons\\Spell_Holy_SealOfSacrifice',
	['Inscription']    = 'Interface\\Icons\\INV_Inscription_Tradeskill01', 
	['Jewelcrafting']  = 'Interface\\Icons\\INV_Misc_Gem_02', 
	['Leatherworking'] = 'Interface\\Icons\\INV_Misc_ArmorKit_17',
--	['Mining']		   = 'Interface\\Icons\\Spell_Fire_FlameBlades',
	['Runeforge']	   = 'Interface\\Icons\\Spell_DeathKnight_FrozenRuneWeapon',
	['Smelting']       = 'Interface\\Icons\\Spell_Fire_FlameBlades',
	['Tailoring']	   = 'Interface\\Icons\\Trade_Tailoring',
}
local LINE_BORDER  = {
	bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background',
	edgeFile = 'Interface\\Tooltips\\UI-Tooltip-Border', edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 },
}

local function sortIndex( a, b)
	if not a then return false end
	if not b then return true end
	if a.index == b.index then 
		return a.name < b.name 
	end
	return a.index < b.index 
end

local function sortLevel( a, b)
	if not a then return false end
	if not b then return true end
	if a.skill == b.skill then 
		return a.name < b.name 
	end
	return a.skill > b.skill
end

local function sortDifficulty( a, b)
	if not a then return false end
	if not b then return true end
	if a.difficulty == b.difficulty then 
		if a.skill == b.skill then 
			return a.name < b.name 
		end
		return a.skill > b.skill
	end
	return a.difficulty < b.difficulty
end

local function sortAlpha( a, b)
	if not a then return false end
	if not b then return true end
	return a.name < b.name 
end

local SORT = {
	['none']       = sortIndex,
	['level']      = sortLevel,
	['difficulty'] = sortDifficulty,
	['alpha']      = sortAlpha,
}
local COLORS = {
	'FF8844',
	'FFFF00',
	'44CC44',
	'AAAAAA',
}

local MAX_INT = 999999

--[[---------------------------------------------------------------------------------
  Local Functions
------------------------------------------------------------------------------------]]
local function PrintLinkToChat( link)
	if ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Insert( link)
	else
		ChatFrame_OpenChat( link)
	end
end

local function DoTrade( data, index, made)
	data.repIndex = index
	data.repCount = tonumber(made)
	DoTradeSkill( index, data.repCount)
end

local function ToggleFarvorite( data, name)
	local f = Producer.db.global.favorites[data.prof] or {}
	if f[name] then
		f[name] = nil
	else
		f[name] = true
	end
	Producer.db.global.favorites[data.prof] = f
end

local function GetMaxMade( reagents)
	local case = MAX_INT
	local bank = MAX_INT
	for _,r in pairs(reagents) do
		if r.link and r.need > 0 then
			local rCase = GetItemCount( r.link, false)
			local rBank = GetItemCount( r.link, true)
			rCase = math.floor(rCase / r.need)
			rBank = math.floor(rBank / r.need)
			case = (rCase < case) and rCase or case
			bank = (rBank < bank) and rBank or bank
		else
			case = 0
			bank = 0
		end
	end
	case = (case < MAX_INT) and case or 0
	bank = (bank < MAX_INT) and bank or 0
	if case > 0 or bank > 0 then
		return format( '[%d/%d]', case, bank - case)
	end
end

-- sell, buy, buyout
local function GetPriceData( link)
	if not link then
		return 0, 0, 0
	end
	if not Producer.db.profile.money then 
		return nil, nil, nil 
	end
	if KC_Items and not KC_Items.common then
		local buyout = select(10, AuctionSpy:GetItemInfo( UnitFactionGroup( 'player'), AuctionSpy:GetLongCode( link)))
		local sell, buy = SellValues:GetPrices( link)
		return sell, buy, buyout
	end
	if KC_Items then
		local resultinfo = KC_Items.common:GetItemInfo( KC_Items.common:GetCode( link))
		local _, _, _, _, _, _, sBuy = KC_Auction:GetItemData( KC_Items.common:GetCode( link, true))
		return resultinfo['sell'], resultinfo['buy'], sBuy
	end
--	if Auctioneer and Auctioneer.HistoryDB and Auctioneer.Util and Auctioneer.ItemDB and Informant then
	if AucAdvanced and Informant then
--		local itemTotals = Auctioneer.HistoryDB.GetItemTotals( Auctioneer.ItemDB.CreateItemKeyFromLink( link), Auctioneer.Util.GetAuctionKey())
--		local avgBuy = 0
--		if itemTotals and itemTotals.buyoutCount > 0 then
--			avgBuy = math.floor( itemTotals.buyoutPrice / itemTotals.buyoutCount)
--		end
		local avgBuy = AucAdvanced.API.GetMarketValue( link)
		local itemid = tonumber( link:match( 'item:(%d+)'))
		local venddata = Informant.GetItem( itemid)
		if venddata then
			return venddata.sell, venddata.buy, avgBuy
		end
		return 0, 0, avgBuy
	end
end

local function round( val) 
	return math.floor( val + 0.5) 
end

local function GetColor( smooth, trade, rank)
	if smooth and trade.skill then
		local percent = trade.skill / rank
		if percent > 1 then
			percent = 1
		end
		if percent >= 0.6667 then
			--RANGE: FF8844 (TOP) to FFFF00 (BOTTOM)
			--  1.000,0.531,0.265 to 1.000,1.000,0.000
			return format( '%02X%02X%02X', 255, round(493 - percent * 357), round(percent * 204 - 136))
		elseif percent >= 0.3333 then
			--RANGE: FFFF00 (TOP) to 44CC44 (BOTTOM)
			--  1.000,1.000,0.000 to 0.265,0.786,0.265
			return format( '%02X%02X%02X', round(percent * 561 - 119), round(153 + percent * 153), round(136 - percent * 204))
		else
			--RANGE: 44CC44 (TOP) to AAAAAA (BOTTOM)
			--  0.265,0.786,0.265 to 0.664,0.664,0.664
			return format( '%02X%02X%02X', round(170 - percent * 306), round(170 + percent * 102), round(170 - percent * 306))
		end
	else
		return COLORS[trade.difficulty]
	end
end

local function GetMoneyText( cash)
	local g = floor( cash/10000)
	local s = mod( floor( cash/100), 100)
	local c = mod( cash, 100)
	if g > 0 then
		return format( '|cFFFFD700%dg|r|cFFC7C7CF%02ds|r|cFFEDA55F%02dc|r', g, s, c)
	elseif s > 0 then
		return format( '|cFFC7C7CF%ds|r|cFFEDA55F%02dc|r', s, c)
	elseif c > 0 then
		return format( '|cFFEDA55F%dc|r', c)
	else
		return ''
	end
end

--[[---------------------------------------------------------------------------------
  Menu Functions
------------------------------------------------------------------------------------]]
local function AddItemLine( trade, grp)
	local v = { token = trade.name }
	v.text = format( '|cFF%s%s|r', COLORS[trade.difficulty], trade.name)
	local count = GetMaxMade( trade.reagents)
	if trade.skill then
		v.second = format( '%s (%s)', count or '', trade.skill)
	else
		v.second = count
	end
	if Producer.db.profile.icons and trade.link then
		v.icon = select( 10, GetItemInfo( trade.link))
	end
	if Producer.db.profile.craftTips then
		v.link = trade.link
	end
	grp:AddLine( v)
end

local CreateCrafted

local function AddItemReagentLine( data, reag, price, grp)
	local v = { token = reag.name }
	local case = reag.link and GetItemCount( reag.link, false) or 0
	local bank = reag.link and GetItemCount( reag.link, true) or 0
	if reag.vendor then
		v.text = format('|cFF6666FF%s|r [%d/%d] (%s)', reag.name, case, reag.need, L.Vendor)
	elseif bank > case then
		v.text = format('|cFF6666FF%s|r [%d/%d] (%s: %d)', reag.name, case, reag.need, L.Bank, bank - case)
	else
		v.text = format('|cFF6666FF%s|r [%d/%d]', reag.name, case, reag.need)
	end
	v.second = price
	local trade = data.profession[reag.name]
	if trade then
		v.hasArrow = true
		v.func = CreateCrafted
		v.args = { data, trade }
	end
	if Producer.db.profile.icons and reag.link then
		v.icon = select( 10, GetItemInfo( reag.link))
	end
	if Producer.db.profile.reagentTips then
		v.link = reag.link
	end
	grp:AddLine( v)
end

local function MakeDBChange( trade, grp)
--	if text ~= L.Options then
--	end
end

local function AddGrouping( trade, grp)
	grp:AddLine()
	grp:AddLine{ text = L.Grouping, hasEditBox = true, editBoxFunc = MakeDBChange, editBoxArg = { trade }, hasArrow = true }
end

function CreateCrafted( data, trade, grp)
	AddItemLine( trade, grp)
	if Producer:GetProfession( data.prof, 'favorite') then
		local f = Producer.db.global.favorites[data.prof] or {}
		grp:AddLine{ text = L.Favorite, func = ToggleFarvorite, args = { data, trade.name }, checked = f[trade.name]  }
	end
	if not trade.cast and Producer.db.profile.money and trade.link then
		local sell, buy, buyout = GetPriceData( trade.link)
		if buyout and buyout > 0 then
			grp:AddLine{ text = L.MarketValue, second = GetMoneyText( buyout), isTitle = true }
		end
		if sell and sell > 0 then
			grp:AddLine{ text = L.VendorValue, second = GetMoneyText( sell), isTitle = true }
		end
	end
	grp:AddLine()
	if not data.isLink then
		if trade.cast then
			grp:AddLine{ text = format( '|cFF00FF00%s|r', trade.cast), func = DoTrade, args = { data, trade.index } }
		else
--			local numMade  = GetTradeSkillNumMade( trade.index)
			grp:AddLine{ text = format( '|cFF00FF00%s|r', L.CreateOne), func = DoTrade, args = { data, trade.index } }
			local val = (trade.index == data.repIndex) and data.repCount or 1
			grp:AddLine{ text = format( '|cFF00FF00%s|r', L.CreateNum), func = DoTrade, args = { data, trade.index }, editNumber = val, editbox = true }
			grp:AddLine{ text = format( '|cFF00FF00%s|r', L.CreateAll), func = DoTrade, args = { data, trade.index, trade.made } }
		end
		grp:AddLine()
		local tools = BuildColoredListString( GetTradeSkillTools( trade.index))
		if tools then
			grp:AddLine{ token = 'Tools', text = tools, isPlain = true }
		end
		local cooldown = GetTradeSkillCooldown( trade.index)
		if cooldown and cooldown > 0 then
			grp:AddLine{ token = 'Cooldown', text = format( '|cFFFF3333%s: %s|r', L.Cooldown, SecondsToTime( cooldown)), isPlain = true }
		end
	end
	local total = 0
	local isTotal = true
	for _,r in pairs(trade.reagents) do
		local price
		if Producer.db.profile.money and r.link then
			local sell, buy, buyout = GetPriceData( r.link)
			if buy and buy > 0 and not (buyout and buyout ~= 0 and Auctioneer) then
				price = buy * r.need
				total = total + price
				price = GetMoneyText( price)
			elseif buyout and buyout > 0 then
				price = buyout * r.need
				total = total + price
				price = GetMoneyText( price)
			else
				price = ''
				isTotal = false
			end
		else
			price = ''
			isTotal = false
		end
		AddItemReagentLine( data, r, price, grp)
	end
	if isTotal and total > 0 then
		grp:AddLine{ text = L.MatsCost, second = GetMoneyText( total), isTitle = true }
	end
	grp:AddLine()
	if trade.link then
		grp:AddLine{ text = format( '|cFFFFFF00%s|r', L.LinkResult), func = PrintLinkToChat, args = { trade.link } }
	end
	if trade.recipe then
		grp:AddLine{ text = format( '|cFFFFFF00%s|r', L.LinkRecipe), func = PrintLinkToChat, args = { trade.recipe } }
	end
--	AddGrouping( trade, grp)
end

local function AddTradeLines( data, trades, grp)
	if table.getn(trades) == 0 then
		grp:AddLine{ text = L.CannotCreate, isTitle = true }
	else
		table.sort( trades, SORT[Producer:GetProfession( data.prof, 'sort')])
		for _,t in ipairs(trades) do
			local v = { token = t.name, func = CreateCrafted, args = { data, t }, justifyH2 = 'CENTER', hasArrow = true }
			v.text = format( '|cFF%s%s|r', GetColor( Producer:GetProfession( data.prof, 'smooth'), t, data.rank), t.name)
			v.second = GetMaxMade( t.reagents)
			if Producer.db.profile.icons and t.link then
				v.icon = select( 10, GetItemInfo( t.link))
			end
			if Producer.db.profile.craftTips then
				v.link = t.link
			end
			grp:AddLine( v)
		end
	end
end

local function GetMinSkill( data)
	local l = tonumber(Producer:GetProfession( data.prof, 'level')) or 0
	return l > 0 and l < data.rank and (data.rank - l) or 0
end

local function GetGroups( data)
	local result = {}
	local added = {}
	for k,v in pairs(data.profession) do
		if v.set and not added[v.set] then
			table.insert( result, v.set)
			added[v.set] = true
		end
	end
	table.sort( result)
	return result
end

local function CreateSet( data, set, grp)
	local minSkill = GetMinSkill( data)
	local trades = {}
	for _,v in pairs(data.profession) do
		if v.set == set and v.skill >= minSkill then
			table.insert( trades, v)
		end
	end
	AddTradeLines( data, trades, grp)
end

local function CreateOranges( data, grp)
	local minSkill = GetMinSkill( data)
	local trades = {}
	for _,v in pairs(data.profession) do
		if v.difficulty == 1 and v.skill >= minSkill then
			table.insert( trades, v)
		end
	end
	AddTradeLines( data, trades, grp)
end

local function CreateYellows( data, grp)
	local minSkill = GetMinSkill( data)
	local trades = {}
	for _,v in pairs(data.profession) do
		if v.difficulty == 2 and v.skill >= minSkill then
			table.insert( trades, v)
		end
	end
	AddTradeLines( data, trades, grp)
end

local function CreateCanMake( data, grp)
	local minSkill = GetMinSkill( data)
	local trades = {}
	for _,v in pairs(data.profession) do
		if v.skill >= minSkill then
			local make = GetMaxMade( v.reagents)
			if make then
				table.insert( trades, v)
			end
		end
	end
	AddTradeLines( data, trades, grp)
end

local function CreateFavorite( data, grp)
	local f = Producer.db.global.favorites[data.prof] or {}
	local trades = {}
	for _,v in pairs(data.profession) do
		if f[v.name] then
			table.insert( trades, v)
		end
	end
	AddTradeLines( data, trades, grp)
end

local function CreateMenu( data, grp)
--	Producer:Debug( 'CreateMenu', title)
	grp:AddLine{ text = data.title, isTitle = true }
	grp:AddLine()
	local trades = GetGroups( data)
	for _,set in pairs(trades) do
		grp:AddLine{ text = set, func = CreateSet, args = { data, set }, hasArrow = true }
	end
	local isOrange = Producer:GetProfession( data.prof, 'orange')
	local isYellow = Producer:GetProfession( data.prof, 'yellow')
	if not data.isLink and (isOrange or isYellow) then
		grp:AddLine()
		if isOrange then
			grp:AddLine{ text = L.Oranges, func = CreateOranges, args = { data }, hasArrow = true }
		end
		if isYellow then
			grp:AddLine { text = L.Yellows, func = CreateYellows, args = { data }, hasArrow = true }
		end
	end
	local isCanMake  = Producer:GetProfession( data.prof, 'canmake')
	local isFavorite = Producer:GetProfession( data.prof, 'favorite')
	if isCanMake or isFavorite then
		grp:AddLine()
		if isCanMake then
			grp:AddLine{ text = L.CanMake, func = CreateCanMake, args = { data }, hasArrow = true }
		end
		if isFavorite then
			grp:AddLine{ text = L.Favorite, func = CreateFavorite, args = { data }, hasArrow = true }
		end
	end
end

--[[---------------------------------------------------------------------------------
  Main
------------------------------------------------------------------------------------]]
local function Control_OnShow()
	PlaySound( 'igCharacterInfoOpen')
end

local function Control_OnHide()
	CloseTradeSkill()
	PlaySound( 'igCharacterInfoClose')
end

local function Control_OnEnter( ctrl, name, frame)
--	Producer:Debug( 'Control_OnEnter')
	if frame.tooltip then
		GameTooltip:SetOwner( frame, 'ANCHOR_TOPLEFT');
		GameTooltip:SetText( frame.tooltip, nil, nil, nil, nil, 1);
		GameTooltip:Show();
	end
end

local function Control_OnLeave()
	GameTooltip:Hide();
end

local function Control_OnMouseDown()
	if CraftFrame.frame:IsMovable() then
		CraftFrame.frame:StartMoving()
	end
end

local function Control_OnMouseUp()
	if CraftFrame.frame:IsMovable() then
		Producer.db.profile.xPos = CraftFrame.frame:GetLeft()
		Producer.db.profile.yPos = CraftFrame.frame:GetBottom()
	end
	CraftFrame.frame:StopMovingOrSizing()
end

local function Control_OnClose()
	CraftFrame:Hide()
end

local function Control_OnLink()
	if ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Insert( GetTradeSkillListLink())
	else
		ChatFrame_OpenChat( GetTradeSkillListLink())
	end
end

local function Control_OnSave( self)
	local saved = not self.userdata.saved
	self:SetSaved( saved)
	Producer:SaveChar( saved)
end

local function Control_OnSwitch()
--	Producer:Debug( 'Control_OnSwitch')
	CraftFrame.menu:CloseNeighbour()
	Producer:SwitchCharOpen( CraftFrame.head)
end

local function Control_OnSelect()
--	Producer:Debug( 'Control_OnSelect')
	CraftFrame.menu:CloseNeighbour()
	Producer:SwitchSelectOpen( CraftFrame.head)
end

--[[---------------------------------------------------------------------------------
  CraftFrame
------------------------------------------------------------------------------------]]
function CraftFrame:Init()
	if self.head then return end
	self:SetLayout( 'ListZoom')
	local head = AceGUI:Create( 'ProducerHead')
	head.width = 'fill'
	head:SetCallback( 'OnClose',  Control_OnClose)
	head:SetCallback( 'OnLink',   Control_OnLink)
	head:SetCallback( 'OnSave',   Control_OnSave)
	head:SetCallback( 'OnSwitch', Control_OnSwitch)
	head:SetCallback( 'OnSelect', Control_OnSelect)
	head:SetCallback( 'OnEnter',  Control_OnEnter)
	head:SetCallback( 'OnLeave',  Control_OnLeave)
	head:SetCallback( 'OnMouseDown', Control_OnMouseDown)
	head:SetCallback( 'OnMouseUp',   Control_OnMouseUp)
	self.head = head
	self:AddChild( head)
	local menu = Snowflake:CreateMain()
	menu.width = 'fill'
	menu:SetCallback( 'OnClose',     Control_OnClose)
	menu:SetCallback( 'OnMouseDown', Control_OnMouseDown)
	menu:SetCallback( 'OnMouseUp',   Control_OnMouseUp)
	self.menu = menu
	self:AddChild( menu)
	local frame = self.frame
	frame:SetMovable( true)
	frame:SetScript( 'OnShow', Control_OnShow)
	frame:SetScript( 'OnHide', Control_OnHide)
end

function CraftFrame:Show()
--	Producer:Debug( 'CraftFrame:Show')
	if not self.head then self:Init() end
	self.frame:Show()
end

function CraftFrame:Hide()
	self.frame:Hide()
	self.menu:CloseNeighbour()
end

function CraftFrame:Update( data, xPos, yPos, locked)
--	Producer:Debug( 'CraftFrame:Update')
	local head = self.head
--	head:SetLogo( data.prof and ICONS_PROFESSION[data.prof])
	head:SetLogo( data.prof and Producer:GetProfessionIcon( data.prof))
	head:SetSelect( data.select or '')
	if data.isLink then
		head:SetLink( false)
		head:SetSaved( data.saved)
		head:SetCharacter( data.player or '', 1, 0, 0)
	else
		head:SetLink( true)
		head:SetSaved( nil)
		head:SetCharacter( data.player or '', 0, 1, 0)
	end
	self.menu:SetFunction( CreateMenu, { data })
	self.menu:Refresh()
	self:DoLayout()
	local frame = self.frame
	frame:ClearAllPoints()
	frame:SetPoint( 'BOTTOMLEFT', frame:GetParent(), 'BOTTOMLEFT', xPos, yPos)
	frame:SetMovable( not locked)
end
