--[[
Producer - a framework for World of Warcraft professions

Copyright (C) 2006-2008  Prat Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Producer = LibStub( 'AceAddon-3.0'):GetAddon( 'Producer')
local L = LibStub( 'AceLocale-3.0'):GetLocale( 'Producer')
	
local SORT_TYPES = {
	['none']       = 1,
	['level']      = 2,
	['difficulty'] = 3,
	['alpha']      = 4,
}
local SORT_DESC = {
	L.SortNone,
	L.SortLevel,
	L.SortDifficulty,
	L.SortAlpha,
}

local function GetProperty( info)
	return Producer:GetProperty( info.arg)
end

local function ToggleProperty( info) 
	Producer:SetProperty( info.arg, not Producer:GetProperty( info.arg))
end

local function IsDebug()
	return Producer:IsDebug()
end

local function ToggleDebug( info) 
	Producer:SetDebug( not Producer:IsDebug())
end

local function GetProfession( info)
	return Producer:GetProfession( info.arg[1], info.arg[2])
end

local function SetProfession( info, value)
	Producer:SetProfession( info.arg[1], info.arg[2], value)
end

local function ToggleProfession( info) 
	Producer:SetProfession( info.arg[1], info.arg[2], not Producer:GetProfession( info.arg[1], info.arg[2]))
end

local function GetNumber( info)
	return tostring(Producer:GetProfession( info.arg[1], info.arg[2]) or '0')
end

local function SetNumber( info, value)
	Producer:SetProfession( info.arg[1], info.arg[2], tonumber(value) or 0)
end

local function PrintVersion() 
	Producer:Print( Producer.version) 
end

local function PrintHelp()
	local function PrintCmd( cmd, desc) Producer:Print( format( ' - |cFF33FF99%s|r: %s', cmd, desc)) end
	Producer:Print( 'Commands (/prod, /producer)')
	PrintCmd( 'enable', L.EnabledDesc)
	PrintCmd( 'lock', L.LockDesc)
	PrintCmd( 'debug', L.DebugDesc)
	PrintCmd( 'version', L.VersionDesc)
end

local function GetSort( info)
	return SORT_TYPES[GetProfession( info)]
end

local function SetSort( info, value)
	value = tonumber(value)
	if value then
		for k,v in pairs(SORT_TYPES) do
			if v == value then
				SetProfession( info, k)
				break
			end
		end
	end
end

local function GetSelections( info)
	return Producer:GetSelections( info.arg)
end

local function GetSelect( info)
	return Producer:GetSelect( info.arg)
end

local function SetSelect( info, value)
	return Producer:SetSelect( info.arg, value)
end

local function ProfessionTable( order, name)
	local prof = Producer:GetProfessionProf( name) or name or 'All'
	local args = {
		name     = { type = 'description', order = 0, name = (name or 'All'), cmdHidden = true },
		space    = { type = 'description', order = 1, name = '',   cmdHidden = true },
		enabled  = { type = 'toggle', order =  2, name = L.EnabledName,  desc = L.EnabledDesc,  arg = { prof, 'enabled'},   width = 'full' },
		oranges  = { type = 'toggle', order =  3, name = L.OrangeName,   desc = L.OrangeDesc,   arg = { prof, 'orange' },   width = 'full' },
		yellows  = { type = 'toggle', order =  4, name = L.YellowName,   desc = L.YellowDesc,   arg = { prof, 'yellow' },   width = 'full' },
		canmake  = { type = 'toggle', order =  5, name = L.CanMakeName,  desc = L.CanMakeDesc,  arg = { prof, 'canmake' },  width = 'full' },
		favorite = { type = 'toggle', order =  6, name = L.FavoriteName, desc = L.FavoriteDesc, arg = { prof, 'favorite' }, width = 'full' },
		smooth   = { type = 'toggle', order =  7, name = L.SmoothName,   desc = L.SmoothDesc,   arg = { prof, 'smooth' },   width = 'full' },
		sort     = { type = 'select', order =  8, name = L.SortName,     desc = L.SortDesc,     arg = { prof, 'sort' },     values = SORT_DESC,  get = GetSort,       set = SetSort },
		level    = { type = 'input',  order = 10, name = L.LevelName,    desc = L.LevelDesc,    arg = { prof, 'level' },    pattern = '^%d*$',   get = GetNumber,     set = SetNumber },
	}
	if name then
		args.select = { type = 'select', order =  9, name = L.SortName, desc = L.SortDesc, arg = prof, values = GetSelections, get = GetSelect, set = SetSelect }
	end
	return { type = 'group', order = order, name = (name or 'All'), get = GetProfession, set = ToggleProfession, args = args }
end

local function TradeTable( order)
	local args = {}
	args['profs'] = ProfessionTable( 0, nil)
	args['space'] = { type = 'group', order = 1,  name = '', cmdHidden = true, args = {} }
	for k,v in pairs(Producer:GetProfessions()) do
		args['prof' .. k] = ProfessionTable( k + 10, v)
	end
	return { type = 'group', order = order, name = L.Trades, args = args } -- dialogHidden = true, dialogInline = true, 
end

local options = {
	type = 'group', name = Producer.title, get = GetProperty, set = ToggleProperty,
	args = {
		release     = { type = 'description', order = 2,  name = Producer.version, cmdHidden = true },
		description = { type = 'description', order = 3,  name = L.Description,    cmdHidden = true },
		
		space1   = { type = 'description', order = 4,  name = '', cmdHidden = true },
		enable   = { type = 'toggle', order = 14, name = L.EnabledName,  desc = L.EnabledDesc, width = 'full', arg = 'enabled' },
		debug    = { type = 'toggle', order = 15, name = L.DebugName,    desc = L.DebugDesc,   width = 'full', get = IsDebug, set = ToggleDebug },
		lock     = { type = 'toggle', order = 16, name = L.LockName,     desc = L.LockDesc,    width = 'full', arg = 'locked' },

		space3   = { type = 'description', order = 30,  name = '', cmdHidden = true },
		level2   = { type = 'toggle', order = 31, name = L.CraftTipsName,   desc = L.CraftTipsDesc,   width = 'full', arg = 'craftTips' },
		level3   = { type = 'toggle', order = 32, name = L.ReagentTipsName, desc = L.ReagentTipsDesc, width = 'full', arg = 'reagentTips' },
		icons    = { type = 'toggle', order = 33, name = L.IconName,        desc = L.IconDesc,        width = 'full', arg = 'icons' },
		money    = { type = 'toggle', order = 35, name = L.MoneyName,       desc = L.MoneyDesc,       width = 'full', arg = 'money' },

		version  = { type = 'toggle', order = 40, name = 'version', dialogHidden = true, disabled = true, get = PrintVersion, set = false },
		help     = { type = 'toggle', order = 41, name = 'help',    dialogHidden = true, disabled = true, get = PrintHelp,    set = false },
--		trades   = TradeTable( 50),
	},
}

local optionProducer
local optionTrades
local optionProfile

local function OnCommand( input)
	if input and input:trim() ~= '' then
		LibStub( 'AceConfigCmd-3.0'):HandleCommand( 'producer', 'producer', input)
	elseif optionProducer and optionProfile then
		InterfaceOptionsFrame_OpenToCategory( optionProfile)
		InterfaceOptionsFrame_OpenToCategory( optionProducer)
	end
end

function Producer:InitConfig()
	local profile = LibStub( 'AceDBOptions-3.0'):GetOptionsTable( self.db)
--	profile.dialogHidden = true
--	profile.dialogInline = true
	profile.args.desc.dialogHidden = true
	local config = LibStub('AceConfig-3.0')
	config:RegisterOptionsTable( 'producer', options)
	config:RegisterOptionsTable( 'producer.trades', TradeTable( 50))
	config:RegisterOptionsTable( 'producer.profile', profile)
	local dialog = LibStub( 'AceConfigDialog-3.0')
	dialog:SetDefaultSize( 'Producer', 600, 400)
	optionProducer = dialog:AddToBlizOptions( 'producer', 'Producer')
	optionTrades   = dialog:AddToBlizOptions( 'producer.trades', L.Trades,  'Producer')
	optionProfile  = dialog:AddToBlizOptions( 'producer.profile', L.Profile, 'Producer')
	self:RegisterChatCommand( 'producer', OnCommand)
	self:RegisterChatCommand( 'prod', OnCommand)
end
