--[[
Snowflake - a library to provide a dropdown menu interface for professions

Copyright (C) 2006-2008  Prat Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local AceGUI = LibStub("AceGUI-3.0")

--[[---------------------------------------------------------------------------------
  Button
------------------------------------------------------------------------------------]]
local Type = 'SnowflakeButton'
local Version = 1
	
local function OnAcquire( self)
end

local function OnRelease( self)
end

local function Control_OnEnter( btn)
	btn.obj:Fire( 'OnEnter')
end

local function Control_OnLeave( btn)
	btn.obj:Fire( 'OnLeave')
end

local function Control_OnClick( btn)
	btn.obj:Fire( 'OnClick')
end

local function Control_OnMouseDown( btn)
	btn.obj:Fire( 'OnMouseDown')
	AceGUI:ClearFocus()
end

local function Control_OnMouseUp( btn)
	btn.obj:Fire( 'OnMouseUp')
end

local function SetSelect( self, value)
	if value and not self.userdata.disabled then
		self.highlight:Show()
	else
		self.highlight:Hide()
	end
end

local function SetInfo( self, info)
	self.userdata.token    = info.token or info.text or self.frame:GetName()
	self.userdata.link     = info.link
	self.userdata.disabled = info.isTitle or info.isPlain or info.disabled
	self.userdata.func     = info.func
	self.userdata.args     = info.args or {}
	self.userdata.hasArrow = info.hasArrow
	self.userdata.doClose  = info.closeWhenClicked
	if info.isTitle then
		self.first:SetFontObject( GameFontNormalSmall)
		self.second:SetFontObject( GameFontNormalSmall)
	elseif info.disabled then
		self.first:SetFontObject( GameFontDisableSmall)
		self.second:SetFontObject( GameFontDisableSmall)
	else
		self.first:SetFontObject( GameFontHighlightSmall)
		self.second:SetFontObject( GameFontHighlightSmall)
	end
	self.image:SetDesaturated( info.disabled)
	if info.checked then
		if info.icon then
			self.image:SetTexture( info.icon)
			self.image:SetWidth( info.iconWidth or 16)
			self.image:SetHeight( info.iconHeight or 16)
		else
			self.image:SetTexture( 'Interface\\Buttons\\UI-CheckBox-Check')
			self.image:SetWidth( 24)
			self.image:SetHeight( 24)
		end
		self.image:SetVertexColor( 1, 1, 1, 1)
		self.image:Show()
	elseif info.icon then
		self.image:SetTexture( info.icon)
		self.image:SetWidth( info.iconWidth or 16)
		self.image:SetHeight( info.iconHeight or 16)
		self.image:SetVertexColor( 1, 1, 1, 1)
		self.image:Show()
	else
		self.image:Hide()
	end
	if info.iconLeft and info.iconRight and info.iconTop and info.iconBottom then
		self.image:SetTexCoord( info.iconLeft, info.iconRight, info.iconTop, info.iconBottom)
	elseif info.icon and info.icon:find( '^Interface\\Icons\\') then
		self.image:SetTexCoord( 0.05, 0.95, 0.05, 0.95)
	else
		self.image:SetTexCoord( 0, 1, 0, 1)
	end
	self.arrow:SetDesaturated( info.disabled)
	self.arrow:SetAlpha( info.hasArrow and 1 or 0)
	local textHeight = info.textHeight or UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT
	self.first:SetFont( STANDARD_TEXT_FONT, textHeight)
	self.first:SetJustifyH( info.justifyH or 'LEFT')
	self.first:SetText( info.text or '')
	self.second:SetFont( STANDARD_TEXT_FONT, textHeight)
	self.second:SetJustifyH( info.justifyH2 or 'RIGHT')
	if info.second then
		self.second:SetText( info.second)
		self.second:Show()
	else
		self.second:SetText( '')
		self.second:Hide()
	end
	self.frame:SetHeight( textHeight + 6)
	self.highlight:Hide()
end

local function GetMinWidth( self)
	local sw = self.second:GetStringWidth()
	if sw > 0 and sw < 50 then
		sw = 50
	end
	return 28 + self.first:GetStringWidth() + sw + 20
end

local function Constructor()
	local id  = AceGUI:GetNextWidgetNum( Type)
	local self = { num = id, type = Type }

	local frame = CreateFrame( 'Button', Type .. id, UIParent)
	frame.obj = self
	frame:SetHeight( 16)
	frame:SetAlpha( 1)
	frame:EnableMouse( true)
	frame:SetFrameStrata( 'FULLSCREEN_DIALOG')
	frame:SetScript( 'OnEnter', Control_OnEnter)
	frame:SetScript( 'OnLeave', Control_OnLeave)
	frame:SetScript( 'OnClick', Control_OnClick)
	frame:SetScript( 'OnMouseDown', Control_OnMouseDown)
	frame:SetScript( 'OnMouseUp',   Control_OnMouseUp)
	self.frame = frame

	local highlight = frame:CreateTexture( nil, 'BACKGROUND')
	highlight:SetTexture( 'Interface\\QuestFrame\\UI-QuestTitleHighlight')
	highlight:SetBlendMode( 'ADD')
	highlight:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	highlight:SetPoint( 'TOPRIGHT', frame, 'TOPRIGHT', 0, 0)
	highlight:Hide()
	self.highlight = highlight

	local image = frame:CreateTexture( nil, 'ARTWORK')
	image:SetTexture( 'Interface\\Buttons\\UI-CheckBox-Check')
	image:SetPoint( 'CENTER', frame, 'LEFT', 12, 0)
	image:SetWidth( 24)
	image:SetHeight( 24)
	image:Hide()
	self.image = image

	local arrow = frame:CreateTexture( nil, 'ARTWORK')
	arrow:SetTexture( 'Interface\\ChatFrame\\ChatFrameExpandArrow')
	arrow:SetPoint( 'RIGHT', frame, 'RIGHT', 0, 0)
	arrow:SetWidth( 16)
	arrow:SetHeight( 16)
	self.arrow = arrow

	local second = frame:CreateFontString( nil, 'ARTWORK')
	second:SetFontObject( GameFontHighlightSmall)
	second:SetFont( STANDARD_TEXT_FONT, UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
	second:SetPoint( 'RIGHT', frame, 'RIGHT', -20, 0)
	second:Hide()
	self.second = second

	local first = frame:CreateFontString( nil, 'ARTWORK')
	first:SetFontObject( GameFontHighlightSmall)
	first:SetFont( STANDARD_TEXT_FONT, UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
	first:SetPoint( 'LEFT', frame, 'LEFT', 28, 0)
	first:SetPoint( 'RIGHT', second, 'LEFT')
	self.first = first

	self.OnAcquire = OnAcquire
	self.OnRelease = OnRelease

	self.SetInfo     = SetInfo
	self.SetSelect   = SetSelect
	self.GetMinWidth = GetMinWidth
	
	AceGUI:RegisterAsWidget( self)
	return self
end
	
AceGUI:RegisterWidgetType( Type, Constructor, Version)
