--[[
Snowflake - a library to provide a dropdown menu interface for professions

Copyright (C) 2006-2008  Prat Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local AceGUI = LibStub("AceGUI-3.0")

--[[---------------------------------------------------------------------------------
Button
------------------------------------------------------------------------------------]]
local Type = 'SnowflakeEditBox'
local Version = 1

local function OnAcquire( self)
end

local function OnRelease( self)
	self:SetSelect( false)
end

local function Control_OnEnter( btn)
	btn.obj:Fire( 'OnEnter')
end

local function Control_OnLeave( btn)
	btn.obj:Fire( 'OnLeave')
end

local function Control_OnClick( btn)
	btn.obj:Fire( 'OnClick')
	btn.obj.box:ClearFocus()
end

local function Control_OnMouseDown( btn)
	btn.obj:Fire( 'OnMouseDown')
	AceGUI:ClearFocus()
end

local function Control_OnMouseUp( btn)
	btn.obj:Fire( 'OnMouseUp')
end

local function Control_OnEnterPressed( box)
	box.obj:Fire( 'OnEnterPressed', box:GetText())
	box:ClearFocus()
end

local function Control_OnEscapePressed( box)
	box:ClearFocus()
end

local function Control_OnEditFocusGained( box)
	box:HighlightText()
	AceGUI:ClearFocus()
end

local function Control_OnEditFocusLost( box)
	box:HighlightText( 0, 0)
end

local function SetSelect( self, value)
	if value and not self.userdata.disabled then
		self.highlight:Show()
	else
		self.highlight:Hide()
	end
end

local function SetInfo( self, info)
	self.userdata.token    = info.token or info.text or self:GetName()
	self.userdata.link     = info.link
	self.userdata.disabled = info.isTitle or info.isPlain or info.disabled
	self.userdata.func     = info.func
	self.userdata.args     = info.args or {}
	self.userdata.doClose  = info.closeWhenClicked
	if info.isTitle then
		self.first:SetFontObject( GameFontNormalSmall)
	elseif info.disabled then
		self.first:SetFontObject( GameFontDisableSmall)
	else
		self.first:SetFontObject( GameFontHighlightSmall)
	end
	self.image:SetDesaturated( info.disabled)
	if info.icon then
		self.image:SetTexture( info.icon)
		self.image:SetWidth( info.iconWidth or 16)
		self.image:SetHeight( info.iconHeight or 16)
		self.image:SetVertexColor( 1, 1, 1, 1)
		self.image:Show()
	else
		self.image:Hide()
	end
	if info.iconLeft and info.iconRight and info.iconTop and info.iconBottom then
		self.image:SetTexCoord( info.iconLeft, info.iconRight, info.iconTop, info.iconBottom)
	elseif info.icon and info.icon:find( '^Interface\\Icons\\') then
		self.image:SetTexCoord( 0.05, 0.95, 0.05, 0.95)
	else
		self.image:SetTexCoord( 0, 1, 0, 1)
	end
	local textHeight = info.textHeight or UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT
	self.first:SetFont( STANDARD_TEXT_FONT, textHeight)
	self.first:SetJustifyH( info.justifyH or 'LEFT')
	self.first:SetText( info.text or '')
	if info.editNumber then
		self.box:SetNumeric( true)
		self.box:SetNumber( info.editNumber)
	else
		self.box:SetNumeric( false)
		self.box:SetText( info.editText)
	end
	self.box:SetMaxLetters( info.maxLetters or 3)
	self.frame:SetHeight( textHeight + 6)
	self.highlight:Hide()
end

local function GetMinWidth( self)
	return 28 + self.first:GetStringWidth() + 44 + 8
end

local function Constructor()
	local id  = AceGUI:GetNextWidgetNum(Type)
	local self = { num = id, type = Type }
	
	local frame = CreateFrame( 'Button', Type .. id, UIParent)
	frame.obj = self
	frame:SetHeight( 16)
	frame:SetAlpha( 1)
	frame:SetFrameStrata( 'FULLSCREEN_DIALOG')
	frame:EnableMouse( true)
	frame:SetScript( 'OnEnter', Control_OnEnter)
	frame:SetScript( 'OnLeave', Control_OnLeave)
--	frame:SetScript( 'OnClick', Control_OnClick)
	frame:SetScript( 'OnMouseDown', Control_OnMouseDown)
	frame:SetScript( 'OnMouseUp', Control_OnMouseUp)
	self.frame = frame
	
	local highlight = frame:CreateTexture( nil, 'BACKGROUND')
	highlight:SetTexture( 'Interface\\QuestFrame\\UI-QuestTitleHighlight')
	highlight:SetBlendMode( 'ADD')
	highlight:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	highlight:SetPoint( 'TOPRIGHT', frame, 'TOPRIGHT', 0, 0)
	highlight:Hide()
	self.highlight = highlight
	
	local image = frame:CreateTexture( nil, 'ARTWORK')
	image:SetTexture( 'Interface\\Buttons\\UI-CheckBox-Check')
	image:SetPoint( 'CENTER', frame, 'LEFT', 12, 0)
	image:SetWidth( 24)
	image:SetHeight( 24)
	image:Hide()
	self.image = image
	
	local box = CreateFrame( 'EditBox', nil, frame)
	box.obj = self
	box:SetFontObject( ChatFontNormal)
	box:SetAutoFocus( false)
	box:SetWidth( 40)
	box:SetHeight( 20)
	box.back = box:CreateTexture( nil, 'BACKGROUND')
	box.back:SetTexture( 'Interface\\COMMON\\Common-CheckBox-Background')
	box.back:SetTexCoord( 0, 1, 0, 0.75)
	box.back:SetAllPoints( box)
	box:SetPoint( 'RIGHT', frame, 'RIGHT', -8, 0)
	box:SetScript( 'OnEnterPressed',    Control_OnEnterPressed)
	box:SetScript( 'OnEscapePressed',   Control_OnEscapePressed)
	box:SetScript( 'OnEditFocusGained', Control_OnEditFocusGained)
	box:SetScript( 'OnEditFocusLost',   Control_OnEditFocusLost)
	self.box = box
	
	local first = frame:CreateFontString( nil, 'ARTWORK')
	first:SetFontObject( GameFontHighlightSmall)
	first:SetFont( STANDARD_TEXT_FONT, UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
	first:SetPoint( 'LEFT', frame, 'LEFT', 28, 0)
	first:SetPoint( 'RIGHT', box, 'LEFT', -4, 0)
	self.first = first
	
	self.OnAcquire  = OnAcquire
	self.OnRelease  = OnRelease

	self.SetInfo     = SetInfo
	self.SetSelect   = SetSelect
	self.GetMinWidth = GetMinWidth
	
	AceGUI:RegisterAsWidget( self)
	return self
end

AceGUI:RegisterWidgetType( Type, Constructor, Version)
